// ---------------------------------------------------------------------
// File: datastructures.h
// Author: Miguel Torres-Torriti (c) 2008.11.21-2009.02.06 v. 1.0
// Change log:
//	2008.11.22, v.1.0, MTT: First (single-file) version was created
//                          using global variables.
//  2009.02.06, v.2.0, MTT: GUI functions were made to that
//                          the input is passed as a pointer
//                          through the optional parameters of 
//                          the callbacks.
// Description:
//  This file provides the data structures used in our lab projects
//  and some functions to handle the associated data structures.
// Remarks:
//  Add shared (public) data structures here.
//
// ---------------------------------------------------------------------
#ifndef DATASTRUCTURES_H
#define DATASTRUCTURES_H

#include <cv.h>
#include <gsl/gsl_matrix.h>

// DATA TYPES AND STRUCTURES

typedef struct bbox {	// Region bounding-box coordinates
	int xmin;
	int ymin;
	int xmax;
	int ymax;
} region;

typedef struct
{
   int    Nfield;           // Number of fields
   region *field;           // Array of Nfield regions
   int    *fieldNelems;     // Number of elements per field
} classdata;

typedef struct
{
	int Nc;					// Number of classes
	classdata *cdataptr;	// Array of Nc classdata structures
} classdataArray;

typedef struct
{
	double prior;
	CvMat* mu;
	CvMat* cov_inv;
	double cov_det;
} classpdf;

typedef struct
{
	int Nc;					// Number of classes
	classpdf *cpdfptr;		// Array of Nc classpdf structures
} classpdfArray;

// ---Array of matrices ---
// Used to create a bank of filter masks
//typedef CvMat* CvMatPtr;
typedef struct { // bank of matrices
	int size;     // bank size
	//CvMatPtr *mask;
	CvMat **mask; // Pointer to CvMat*, to which an array of matrix pointer must be allocated.
} mbank;

typedef struct
{
   // Specific to Gabor feature generation method.
   CvMat *filterbank;
   int filterXlen;
   int filterYlen;
   // Common to all feature generation methods.
   char fmethod;        // 0 GMRF, 1 Gabor, ...
   unsigned int Nf;     // dimension of the feature vector
} fgp, *fgpPtr;

int matrix_show(CvMat* m);
classpdfArray* createclasspdfArray(int Nc, int fv_dim);
void releaseclasspdfArray(classpdfArray* cpdf);
void releasembank(mbank* mb);

#endif