#include "gui.h"
#include "process.h"

// --- GUI FUNCTIONS --- //

char wndname[]  = "Original Image",
	 wndname2[] = "Output Image",
	 wndname3[] = "Edges";

char tbarname2[] = "Threshold";
char tbarname3[] = "Edges";
int cr_thresh = 10;
int edge_thresh = 1;

CvPoint mpos = cvPoint(-1, -1);			// Current mouse position
CvPoint mpos_prev = cvPoint(-1, -1);	// Previous mouse position

CvScalar color_table[20] = {CV_RGB(255,0,0),     // R
							CV_RGB(0,255,0),     // G
							CV_RGB(0,0,255),     // B
							CV_RGB(0,255,255),   // C = G+B
							CV_RGB(255,0,255),   // M = R+B
							CV_RGB(255,255,0),   // Y = R+G
							CV_RGB(255,127,127), // M+Y
							CV_RGB(127,255,127), // C+Y
							CV_RGB(127,127,255), // C+M
							CV_RGB(170,255,255), // 2C+M+Y
							CV_RGB(255,170,255), // C+2M+Y
							CV_RGB(255,255,170), // C+M+2Y
							CV_RGB(255,213,213), // 2C+3M+3Y
							CV_RGB(213,255,213), // 3C+2M+3Y
							CV_RGB(213,213,213), // 3C+3M+2Y
							CV_RGB(232,255,255), // 6C+5M+5Y
							CV_RGB(255,232,255), // 5C+6M+5Y
							CV_RGB(255,255,232), // 5C+5M+6Y
							CV_RGB(255,255,255), // WHITE
							CV_RGB(255,255,232)  // BLACK
						};

bbox rect = {0, 0, 0, 0}; // stores bounding box for user-selected regions

double bboxarea(bbox rect)
{
	return (double)  ((rect.xmax-rect.xmin)*(rect.ymax-rect.ymin));
}

void on_mouse( int event, int x, int y, int flags, void* param )
{
	CvImage* img = (CvImage*) param;
	static CvImage imgAux; // provide an internal permanent storage
						   // not destroyed between calls to this function

	if( img->origin() )
		y = img->height() - y;

	if (event == CV_EVENT_LBUTTONDOWN)
	{
		CvScalar Iv;
		mpos = cvPoint(x,y);

		Iv = cvGet2D( *img, mpos.y, mpos.x );
		printf("(%d, %d) - %d : %d : %d\n", mpos.x, mpos.y, (int) Iv.val[0], (int) Iv.val[1], (int) Iv.val[2]);
    }
	else if (event == CV_EVENT_RBUTTONUP) // || !(flags & CV_EVENT_FLAG_RBUTTON))
	{
		if (mpos.x<rect.xmin)
		{
			rect.xmax=rect.xmin;
			rect.xmin=mpos.x;
		}
		else
		{
			rect.xmax=mpos.x;
		}
		if (mpos.y<rect.ymin)
		{
			rect.ymax=rect.ymin;
			rect.ymin=mpos.y;
		}
		else
		{
			rect.ymax=mpos.y;
		}
		printf("bbox: [%d, %d]x[%d,%d]\n", rect.xmin, rect.ymin, rect.xmax, rect.ymax);

		mpos_prev = cvPoint(-1,-1);
	}
	else if (event == CV_EVENT_RBUTTONDOWN)
	{
		mpos_prev =  cvPoint(x,y);
		mpos = cvPoint(x,y);

		rect.xmin = mpos_prev.x;
		rect.ymin = mpos_prev.y;
		rect.xmax = mpos.x;
		rect.ymax = mpos.y;
		if (flags & CV_EVENT_FLAG_CTRLKEY) // If CTRL key is pressed, copy img to imgAux for painting
			imgAux = img->clone();
	}
	else if (event == CV_EVENT_MOUSEMOVE && (flags & CV_EVENT_FLAG_RBUTTON) && !(flags & CV_EVENT_FLAG_CTRLKEY))
	{   // draw rectangle of selected region
		mpos = cvPoint(x,y);
		if (mpos_prev.x < 0)
			mpos_prev = cvPoint(x,y);

		imgAux = img->clone();
		//cvLine( imgAux, mpos_prev, mpos, cvScalarAll(255), 5, 8, 0 );
		cvRectangle( imgAux, mpos_prev, mpos, cvScalarAll(255), 1, 8, 0 );

		//mpos_prev = mpos;
		cvShowImage( wndname, imgAux );
	}
	else if (event == CV_EVENT_MOUSEMOVE && (flags & CV_EVENT_FLAG_RBUTTON) && (flags & CV_EVENT_FLAG_CTRLKEY))
	{   // paint line on image
		mpos = cvPoint(x,y);
		if (mpos_prev.x < 0)
			mpos_prev = cvPoint(x,y);

		cvLine( imgAux, mpos_prev, mpos, cvScalarAll(255), 5, 8, 0 );

		mpos_prev = mpos;
		cvShowImage( wndname, imgAux );
	}
}

// Define a trackbar callback
void on_trackbar2(int h)
{
	//DetectClasses( &cpar );
	//img_aux.show( wndname );
}

void on_trackbar3(int h)
{
	//img_aux.show( wndname );
}