// ---------------------------------------------------------------------
// File: process.h
// Author: Miguel Torres-Torriti (c) 2008.11.21-2009.02.06 v. 1.0
// Change log:
//	2008.11.22, v.1.0, MTT: First (single-file) version was created
//                          using global variables.
//  2009.02.06, v.2.0, MTT: Process functions were made so that
//                          the input is passed as a pointer.
//                          However the output is locally defined
//                          and shared as extern variables.
// Description:
//  Place your program process routines in this file.
//  This example performs deterministic color-based segmentation.
//  See the .cpp file for details.
// Remarks:
//  Avoid using global variables for process functions.
//  Pass input data and output storage as pointers!
//
// ---------------------------------------------------------------------
#ifndef PROCESS_H
#define PROCESS_H

#include "cv.h"       // use " " instead of < > to get list of class/struct members
#include "mathlib.h"
#include "datastructures.h"
#include "util.h"		// for cmdlinestruct

// --- PROCESS GLOBAL/PUBLIC VARIABLES --- //

typedef struct {	// Classes Data
	unsigned int Nc;	// number of classes 
 	unsigned int c;		// current (pointed class) class
	unsigned int fdim;	// dimension of feature vector
	mmatrix *fv;		// feature data values
} cdatastruct;

extern cdatastruct cpar;

extern CvImage 
		//img,		// Original image (is now defined as static within main)
		imgIRB,		// YCrCb img (Intensity - Red Chrominance - Blue Chrominance)
		imgSegMask,	// Segmentation mask (image)
		imgSeg,		// Segmentation image (original image and segmented pixels)
		imgG;		// Gray scale image

extern bbox rect; // Stores bounding box for user-selected regions

// --- PROCESS PUBLIC FUNCTIONS --- //

CvScalar ColorTransform(CvScalar color_in, int code);
mvector* ComputeFeature(CvImage *img );
void ComputeSegmentation(CvImage *img, cdatastruct *cpar);
void AddClass(cdatastruct *cpar);
void RemoveClass(cdatastruct *cpar);
void PrintClass(cdatastruct *cpar);
void UpdateClass(cdatastruct *cpar);
void RandomlyFillClass(cdatastruct *cpar);
void BlankClass(cdatastruct *cpar);
void DetectClasses(CvImage* img, cdatastruct *cpar);
void IncrementClass(cdatastruct *cpar);
void DecrementClass(cdatastruct *cpar);
void WriteCPD(cdatastruct *cpar, char* filename); // Write Class Parameters Data
void ReadCPD(cdatastruct *cpar, char* filename); // Read Class Parameters Data

int ProcessColorSegmentation(CvImage *img, cmdlinestruct cmdlineparam);
int ProcessColorConversion(CvImage *img);

#endif